@@ -1,6 +1,6 @@
 
 /*
- * $Id: pass.c,v 1.29.2.3 1997/03/03 17:09:25 wessels Exp $
+ * $Id: pass.c,v 1.29.2.5 1997/04/24 19:14:02 wessels Exp $
  *
  * DEBUG: section 39    HTTP Passthrough
  * AUTHOR: Duane Wessels
@@ -48,6 +48,7 @@ typedef struct {
     time_t timeout;
     int *size_ptr;		/* pointer to size for logging */
     int proxying;
+    int ip_lookup_pending;
 } PassStateData;
 
 static void passLifetimeExpire _PARAMS((int fd, void *));
@@ -106,6 +107,8 @@ passStateFree(int fd, void *data)
 	return;
     if (fd != passState->server.fd)
 	fatal_dump("passStateFree: FD mismatch!\n");
+    if (passState->ip_lookup_pending)
+	ipcache_unregister(passState->host, passState->server.fd);
     if (passState->client.fd > -1) {
 	commSetSelect(passState->client.fd,
 	    COMM_SELECT_READ,
@@ -341,6 +344,7 @@ passConnect(int fd, const ipcache_addrs * ia, void *data)
     PassStateData *passState = data;
     request_t *request = passState->request;
     char *buf = NULL;
+    passState->ip_lookup_pending = 0;
     if (ia == NULL) {
 	debug(39, 4, "passConnect: Unknown host: %s\n", passState->host);
 	buf = squid_error_url(passState->url,
@@ -473,6 +477,7 @@ passStart(int fd,
 	NULL, 0);
     if (Config.firewall_ip_list) {
 	/* must look up IP address */
+	passState->ip_lookup_pending = 1;
 	ipcache_nbgethostbyname(passState->host,
 	    passState->server.fd,
 	    passSelectNeighbor,
@@ -494,14 +499,18 @@ passSelectNeighbor(int u1, const ipcache_addrs * ia, void *data)
     peer *e = NULL;
     peer *g = NULL;
     int fw_ip_match = IP_ALLOW;
+    int inside_fw = matchInsideFirewall(request->host);
+    passState->ip_lookup_pending = 0;
     if (ia && Config.firewall_ip_list)
 	fw_ip_match = ip_access_check(ia->in_addrs[ia->cur], Config.firewall_ip_list);
-    if (matchInsideFirewall(request->host)) {
+    if (inside_fw == INSIDE_FIREWALL) {
 	hierarchyNote(request, HIER_DIRECT, 0, request->host);
     } else if (fw_ip_match == IP_DENY) {
 	hierarchyNote(request, HIER_FIREWALL_IP_DIRECT, 0, request->host);
     } else if ((e = Config.passProxy)) {
 	hierarchyNote(request, HIER_PASS_PARENT, 0, e->host);
+    } else if (inside_fw == NO_FIREWALL) {
+	hierarchyNote(request, HIER_DIRECT, 0, request->host);
     } else if ((e = getDefaultParent(request))) {
 	hierarchyNote(request, HIER_DEFAULT_PARENT, 0, e->host);
     } else if ((e = getSingleParent(request))) {
@@ -524,6 +533,7 @@ passSelectNeighbor(int u1, const ipcache_addrs * ia, void *data)
     } else {
 	passState->port = CACHE_HTTP_PORT;
     }
+    passState->ip_lookup_pending = 1;
     ipcache_nbgethostbyname(passState->host,
 	passState->server.fd,
 	passConnect,
