@@ -1,6 +1,6 @@
 
 /*
- * $Id: ssl.c,v 1.38.2.2 1997/03/03 17:09:28 wessels Exp $
+ * $Id: ssl.c,v 1.38.2.5 1997/04/24 19:14:05 wessels Exp $
  *
  * DEBUG: section 26    Secure Sockets Layer Proxy
  * AUTHOR: Duane Wessels
@@ -46,6 +46,7 @@ typedef struct {
     time_t timeout;
     int *size_ptr;		/* pointer to size in an icpStateData for logging */
     int proxying;
+    int ip_lookup_pending;
 } SslStateData;
 
 static const char *const conn_established = "HTTP/1.0 200 Connection established\r\n\r\n";
@@ -117,6 +118,8 @@ sslStateFree(int fd, void *data)
     safe_free(sslState->client.buf);
     xfree(sslState->url);
     requestUnlink(sslState->request);
+    if (sslState->ip_lookup_pending)
+	ipcache_unregister(sslState->host, sslState->server.fd);
     safe_free(sslState);
 }
 
@@ -322,6 +325,7 @@ sslConnect(int fd, const ipcache_addrs * ia, void *data)
     SslStateData *sslState = data;
     request_t *request = sslState->request;
     char *buf = NULL;
+    sslState->ip_lookup_pending = 0;
     if (ia == NULL) {
 	debug(26, 4, "sslConnect: Unknown host: %s\n", sslState->host);
 	buf = squid_error_url(sslState->url,
@@ -444,14 +448,19 @@ sslStart(int fd, const char *url, request_t * request, char *mime_hdr, int *size
 	(void *) sslState);
 
     if (Config.sslProxy) {
-	ipcache_nbgethostbyname(request->host,
+	sslState->host = request->host;
+	/* set sslState->host = request->host so that we can
+	 * cancel it later if needed */
+	sslState->ip_lookup_pending = 1;
+	ipcache_nbgethostbyname(sslState->host,
 	    sslState->server.fd,
 	    sslSelectNeighbor,
 	    sslState);
     } else {
 	sslState->host = request->host;
 	sslState->port = request->port;
-	ipcache_nbgethostbyname(request->host,
+	sslState->ip_lookup_pending = 1;
+	ipcache_nbgethostbyname(sslState->host,
 	    sslState->server.fd,
 	    sslConnect,
 	    sslState);
@@ -487,14 +496,18 @@ sslSelectNeighbor(int fd, const ipcache_addrs * ia, void *data)
     peer *e = NULL;
     peer *g = NULL;
     int fw_ip_match = IP_ALLOW;
+    int inside_fw = matchInsideFirewall(request->host);
+    sslState->ip_lookup_pending = 0;
     if (ia && Config.firewall_ip_list)
 	fw_ip_match = ip_access_check(ia->in_addrs[ia->cur], Config.firewall_ip_list);
-    if (matchInsideFirewall(request->host)) {
+    if (inside_fw == INSIDE_FIREWALL) {
 	hierarchyNote(request, HIER_DIRECT, 0, request->host);
     } else if (fw_ip_match == IP_DENY) {
 	hierarchyNote(request, HIER_FIREWALL_IP_DIRECT, 0, request->host);
     } else if ((e = Config.sslProxy)) {
 	hierarchyNote(request, HIER_SSL_PARENT, 0, e->host);
+    } else if (inside_fw == NO_FIREWALL) {
+	hierarchyNote(request, HIER_DIRECT, 0, request->host);
     } else if ((e = getDefaultParent(request))) {
 	hierarchyNote(request, HIER_DEFAULT_PARENT, 0, e->host);
     } else if ((e = getSingleParent(request))) {
@@ -517,6 +530,7 @@ sslSelectNeighbor(int fd, const ipcache_addrs * ia, void *data)
     } else {
 	sslState->port = CACHE_HTTP_PORT;
     }
+    sslState->ip_lookup_pending = 1;
     ipcache_nbgethostbyname(sslState->host,
 	sslState->server.fd,
 	sslConnect,
