@@ -1,5 +1,5 @@
 /*
- * $Id: unlinkd.c,v 1.1.2.12 1997/04/22 15:26:20 wessels Exp $
+ * $Id: unlinkd.c,v 1.1.2.13 1997/04/28 18:14:17 wessels Exp $
  *
  * DEBUG: section 43    Unlink Daemon
  * AUTHOR: Duane Wessels
@@ -35,6 +35,10 @@ static char hello_string[] = "hi there\n";
 /* This is the external unlinkd process */
 
 #include "config.h"
+
+#if HAVE_LIBC_H
+#include <libc.h>
+#endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
